/**
 * @file		data_builders.c
 * @date		23/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		Construct converted datas structure from extracted raw HMP.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "RSPTerrain_errordefs.h"
#include "RSPTerrain_datatypes.h"
#include "data_builders.h"


unsigned short RSP_TerrainLib_ConstructHeightmap(T_RSPTERRAIN_HEIGHTMAP* pHeightMap, const T_RSPTERRAIN_HMP* pHmpStruct) {
	unsigned int i,j,k,l;

	pHeightMap->height = pHmpStruct->height;
	pHeightMap->width = pHmpStruct->width;

	// Build 2D array heightmap
	pHeightMap->heightmap = (HEIGHTMAP_T)malloc(pHmpStruct->width * RSPTERRAINLIB_TILE_SAMPLING * sizeof(unsigned char *));
	for ( i = 0; i < pHmpStruct->width * RSPTERRAINLIB_TILE_SAMPLING; i++ ) {
		pHeightMap->heightmap[i] = (unsigned char *)malloc(pHmpStruct->height * RSPTERRAINLIB_TILE_SAMPLING * sizeof(unsigned char));
	}

	if (pHeightMap->heightmap) {
		// Select tile
		for ( i = 0; i < pHmpStruct->width; i++ ) {
			for ( j = 0; j < pHmpStruct->height; j++ ) {

				// Get the 5x5 bytes height values for this tile
				for ( k = 0; k < RSPTERRAINLIB_TILE_SAMPLING; k++ ) {
					for ( l = 0; l < RSPTERRAINLIB_TILE_SAMPLING; l++ ) {
						pHeightMap->heightmap[i*RSPTERRAINLIB_TILE_SAMPLING+k][j*RSPTERRAINLIB_TILE_SAMPLING+l] = pHmpStruct->tilesmap[i * pHmpStruct->height + j].height_values[l][k];
					}
				}

			}
		}
	} else return RSPLIB_ERROR_MEMORY;

	return RSPLIB_SUCCESS;
}

unsigned short RSP_TerrainLib_ConstructMesh(T_RSPTERRAIN_MESH* pMesh, const T_RSPTERRAIN_HMP* pHmpStruct, const float meshScale) {
	unsigned int i,j,k,m,n;
	int vertices_width, vertices_height;
	float tmp_x,tmp_z;

	if (pMesh == NULL || pHmpStruct == NULL) return RSPLIB_ERROR_ARGS_NULL;
	if (pHmpStruct->tilesmap == NULL) return RSPLIB_ERROR_ARGS_NULL;

	vertices_width = pHmpStruct->width * RSPTERRAINLIB_TILE_SAMPLING;
	vertices_height = pHmpStruct->height * RSPTERRAINLIB_TILE_SAMPLING;

	pMesh->height = vertices_height;
	pMesh->width = vertices_width;
	pMesh->vertices_count = vertices_width * vertices_height;

	// Build vertex list of the terrain
	pMesh->verticesmap = (T_RSPTERRAIN_VERTEX *)malloc(pMesh->vertices_count * sizeof(T_RSPTERRAIN_VERTEX));

	if (pMesh->verticesmap) {

		// Select tile
		for ( i = 0; i < pHmpStruct->height; i++ ) {
			for ( j = 0; j < pHmpStruct->width; j++ ) {

				k = j * RSPTERRAINLIB_TILE_SAMPLING + i * RSPTERRAINLIB_TILE_SAMPLING * vertices_width;

				for ( n = 0; n < RSPTERRAINLIB_TILE_SAMPLING; n++ ) {
					for ( m = 0; m < RSPTERRAINLIB_TILE_SAMPLING; m++ ) {
						// Terrain center defined as mesh center -- sould not be used for game application
						tmp_x = -(vertices_width / 2);
						tmp_x = meshScale * (tmp_x + j * RSPTERRAINLIB_TILE_SAMPLING + m);
						pMesh->verticesmap[k + n*vertices_width + m].x = tmp_x;
						tmp_z = -(vertices_height / 2);
						tmp_z = meshScale * (tmp_z + i * RSPTERRAINLIB_TILE_SAMPLING + n);
						pMesh->verticesmap[k + n*vertices_width + m].z = tmp_z;

						// Convert read scale to display scale, don't known the real operation in game
						pMesh->verticesmap[k + n*vertices_width + m].y = pHmpStruct->tilesmap[i * pHmpStruct->width + j].height_values[m][n] * (pHmpStruct->height_scale * 2.0 / 10.0);
					}
				}
			}
		}

	} else return RSPLIB_ERROR_MEMORY;

	return RSPLIB_SUCCESS;
}
